/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------
//#include "analyzer.h"

using namespace std;
using namespace Moses1;

#define PDFSETS 41

static TH1 *invMass[PDFSETS];
static TH1 *Rapidity[PDFSETS];

int main() {
	cout << "Holy Moses..." << endl;
	
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();

	
	// declare some histos and related root entities
	commonOperations commonOps;
	string outFileName = "/PDFstudy/CTEQ6/KK_PDFsUncertainty_CTEQ6";
	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".eps");

	TFile *histFile = new TFile(rootFileName.c_str(),"recreate");
	TCanvas *c1 = new TCanvas("c1","Kaluza-Klein",0,0,1400,800);
	c1->Divide(1,2);

	// plots of analysis on muon pairs
	TVirtualPad *pad1=c1->cd(1);
	pad1->SetFillColor(10);
	pad1->SetLogy();
	//pad1->SetGrid();

        TVirtualPad *pad2=c1->cd(2);
        pad2->SetFillColor(10);
        pad2->SetLogy();
        //pad2->SetGrid();


	// boundries and bins of histograms
	Int_t    invMnBins = 40;
	Double_t invMmin   = 1000.;
	Double_t invMmax   = 6000.;

	Int_t rapnBins  = 36;
	Double_t rapMin = -3.;
	Double_t rapMax = 3.;

	//----------------------------------------------------------------------------------
	// logarithmic boundries and bins of histograms
	Double_t logMmin = log10(invMmin);
	Double_t logMmax = log10(invMmax);
	Double_t M_binwidth = (Double_t)( (logMmax-logMmin)/(Double_t)invMnBins );
	Double_t M_bins[invMnBins+1];
	M_bins[0] = invMmin;
	for (Int_t i=1 ; i<=invMnBins ; i++) { M_bins[i] = TMath::Power( 10,(logMmin + i*M_binwidth) ); }
        //----------------------------------------------------------------------------------


	// declaration of histograms
	// invariant mass:
	// declare the 0th histo (central PDF subset)
	invMass[0] = new TH1D("m(mu-,mu+) using central set", "muon pairs invariant mass: PDFs Uncertainty", invMnBins, M_bins);
	invMass[0]->SetLineColor(kRed);
	invMass[0]->SetLineWidth(1);
	invMass[0]->SetXTitle("invariantMass(mu-,mu+)[GeV]");
	invMass[0]->SetYTitle("Events/bin");
	invMass[0]->SetTitle("Muon pairs invariant mass distribution: PDFs Uncertainty");
	invMass[0]->SetMarkerStyle(20);
	invMass[0]->SetMarkerColor(kRed);

	// declare the rest of the 40 histos (rest of PDFs subsets)
	string subset, statTitle, mainTitle;
	stringstream strstm;
	for (int i=1 ; i<PDFSETS ; i++) {
		strstm.clear();
		strstm << i;
		strstm >> subset;
		mainTitle = "PDF subset_" + subset;
		statTitle = "m(mu-,mu+) PDF subset_" + subset; 
		invMass[i] = new TH1D(statTitle.c_str(), mainTitle.c_str(), invMnBins, M_bins);
	}
	// rapidity:
	// declare the 0th histo (central PDF subset)
	Rapidity[0] = new TH1D("y(mu-,mu+) using central set", "muon pairs rapidity: PDFs Uncertainty", rapnBins, rapMin, rapMax);
	Rapidity[0]->SetLineColor(kRed);
	Rapidity[0]->SetLineWidth(1);
	Rapidity[0]->SetXTitle("Rapidity(mu-,mu+)");
	Rapidity[0]->SetYTitle("Events");
	Rapidity[0]->SetTitle("Muon pairs rapidity distribution: PDFs Uncertainty");
	Rapidity[0]->SetMarkerStyle(20);
	Rapidity[0]->SetMarkerColor(kRed);
	// declare the rest of the 40 histos (rest of PDFs subsets)
	for (int i=1 ; i<PDFSETS ; i++) {
		strstm.clear();
		strstm << i;
		strstm >> subset;
		mainTitle = "PDF subset_" + subset;
		statTitle = "y(mu-,mu+) PDF subset_" + subset;
		Rapidity[i] = new TH1D(statTitle.c_str(), mainTitle.c_str(), rapnBins, rapMin, rapMax);
	}

	//----------------------------------------------------------------------------------------------------------
	// declare the relevant classes
	dileptonAnalyzer anlz;
	
	// set cuts
	double invMassCut=1000.;
	 double pTransverseCut=20.;
	double EtaCut=2.5;
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);
	
	// start loop over events file(/files)
	string filePath;
	int Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	int Nleptons, Naleptons;
	double invariantMass;
	double rap;
	string fileName;	

	// loop over PDFSETS files (0 = centralSet, 1->40 = errorSubsets)
	for (int f=0 ; f<PDFSETS ; f++) {
		strstm.clear();
                strstm << f;
                strstm >> subset;
		fileName = "/PDFstudy/CTEQ6/KK_extractedParticles_subset" + subset + ".dat"; 
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", fileName); 
		
		Nevents = anlz.loopOverEvents(filePath);
		cout << "Nevents = " << Nevents << endl;

		// get all the good pairs (after cuts)
		Nleptons = anlz.getAllLeptons(allgoodLeptons);
		Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
		cout << "Nleptons = " << Nleptons << endl;
		if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
	
		// run over the good pairs and fill the histograms
		for (int k=0 ; k<Nleptons ; k++) {
			invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
			rap = anlz.y(allgoodLeptons[k], allgoodaLeptons[k]);
			
			invMass[f]->Fill( (Double_t)invariantMass );
			Rapidity[f]->Fill( (Double_t)rap );
		}
	
		// clear the pairs vectors
		anlz.clearAllLeptons();
		if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
		if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
	}
	//----------------------------------------------------------------------------------------------------------


	//----------------------------------------------------------------------------------------------------------
	// find the uncertainty envelope for the ***invariant mass*** distribution obtained with the central pdfSet
	// allocate arrays for the envelopes
	Double_t *max_invMass = (Double_t *)malloc(invMnBins*sizeof(Double_t));
	Double_t *min_invMass = (Double_t *)malloc(invMnBins*sizeof(Double_t));

	// declare the up/down envelopes histos
	//TH1 *invMass_envelope_up = new TH1D("invMass upEnvelope", "invMass upEnvelope", invMnBins, invMmin, invMmax);
	TH1 *invMass_envelope_up = new TH1D("invMass upEnvelope", "invMass upEnvelope", invMnBins, M_bins);
	invMass_envelope_up->SetLineWidth(2);
	invMass_envelope_up->SetLineColor(kGreen);
	invMass_envelope_up->SetFillColor(3); // green
	invMass_envelope_up->SetXTitle("invariantMass(mu-,mu+)[GeV]");
	invMass_envelope_up->SetYTitle("Events/bin");
	invMass_envelope_up->SetTitle("Muon pairs invariant mass distribution: PDFs Uncertainty");

	TH1 *invMass_envelope_down = new TH1D("invMass downEnvelope", "invMass downEnvelope", invMnBins, M_bins);
	invMass_envelope_down->SetLineWidth(2);
	invMass_envelope_down->SetLineColor(kGreen);
	invMass_envelope_down->SetFillColor(10); // white


	// initialize arrays for the envelope graph
	for (int j=0 ; j<(int)invMnBins ; j++) { max_invMass[j] = min_invMass[j] = 0; }
//	for (int j=0 ; j<(int)invMnBins ; j++) { max_invMass[j] = 0.;   min_invMass[j] = 100000.; }

	// loop over sets
	Double_t binContent, centralSetBinContent;
	Double_t diff;
	int n;
	for (int f=0 ; f<PDFSETS ; f++) {
		n=0;
		while (n<=(int)invMnBins) {
			binContent = (Double_t)invMass[f]->GetBinContent(n);
			centralSetBinContent = (Double_t)invMass[0]->GetBinContent(n);
			diff = centralSetBinContent - binContent;
			if (diff<0) { max_invMass[n] += diff*diff; }
			else        { min_invMass[n] += diff*diff; }
//			if (binContent > max_invMass[n]) { max_invMass[n] = binContent; }
//			if (binContent < min_invMass[n]) { min_invMass[n] = binContent; }
			n++;
		}
	}

	// fill the envelopes histos
	Double_t up, down;
	for (int j=0 ; j<(int)invMnBins ; j++) {
		up = (Double_t)invMass[0]->GetBinContent(j) + (Double_t)sqrt(max_invMass[j]);
		down = (Double_t)invMass[0]->GetBinContent(j) - (Double_t)sqrt(min_invMass[j]);
		if (down<0.) { down = 0.; }
//		up = (Double_t)invMass[0]->GetBinContent(j) + (Double_t)max_invMass[j];
//		down = (Double_t)invMass[0]->GetBinContent(j) - (Double_t)min_invMass[j];

		invMass_envelope_up->SetBinContent((Int_t)j,up);
		invMass_envelope_down->SetBinContent((Int_t)j,down);
	}

	// legends
	//TLegend *leg1 = new TLegend(0.8,0.7,0.98,0.85);
	TLegend *leg1 = new TLegend(0.65,0.68,0.98,0.88);
	leg1->AddEntry(invMass[0],"Central Prediction [cteq6m.LHpdf]", "L");
	leg1->AddEntry(invMass_envelope_up,"Uncertainty Band [cteq6.LHpdf(1->40)]");
	leg1->SetFillColor(5);

/*
	// text box
	//TPaveText *paveText = new TPaveText(0.8,0.4,0.98,0.65, "NDC"); // NDC sets coords relative to pad dimensions
	TPaveText *paveText = new TPaveText(0.13,0.75,0.365,0.89, "NDC"); // NDC sets coords relative to pad dimensions
	paveText->SetFillColor(kWhite);
	paveText->SetTextSize(0.04);
	paveText->SetTextAlign(12);
	TText *t1 = paveText->AddText("Central set: cteq6m.LHpdf");
	TText *t2 = paveText->AddText("Envelope sets(1->40): cteq6.LHpdf");
	t1->SetTextColor(kRed);
	t2->SetTextColor(kGreen);
*/
	// divide by the bin width
	Double_t binSize, binValue, binError;
	for (Int_t b=1 ; b<=invMnBins ; b++) {
		binValue = invMass[0]->GetBinContent(b);
		binError = invMass[0]->GetBinError(b);
		binSize = invMass[0]->GetBinLowEdge(b+1) - invMass[0]->GetBinLowEdge(b);
		invMass[0]->SetBinContent(b,binValue/binSize);
		invMass[0]->SetBinError(b,binError/binSize);
	
		binValue = invMass_envelope_up->GetBinContent(b);
		//binError = invMass_envelope_up->GetBinError(b);
		binSize = invMass_envelope_up->GetBinLowEdge(b+1) - invMass_envelope_up->GetBinLowEdge(b);
		invMass_envelope_up->SetBinContent(b,binValue/binSize);
		//invMass_envelope_up->SetBinError(b,binError/binSize);

		binValue = invMass_envelope_down->GetBinContent(b);
		//binError = invMass_envelope_down->GetBinError(b);
		binSize = invMass_envelope_down->GetBinLowEdge(b+1) - invMass_envelope_down->GetBinLowEdge(b);
		invMass_envelope_down->SetBinContent(b,binValue/binSize);
		//invMass_envelope_down->SetBinError(b,binError/binSize);
	}
	
	// reset the number of entries
	invMass[0]->SetEntries( invMass[0]->GetEntries() - invMnBins );
	invMass_envelope_up->SetEntries( invMass_envelope_up->GetEntries() - invMnBins );
	invMass_envelope_down->SetEntries( invMass_envelope_down->GetEntries() - invMnBins );

	// draw the histos
	c1->cd(1);
	gStyle->SetOptStat(11);
	invMass_envelope_up->Draw();
	invMass_envelope_down->Draw("sames");
	invMass[0]->DrawCopy("e1sames");
	invMass[0]->Draw("sames");
	leg1->Draw("sames");
	//paveText->Draw("sames");
	gPad->RedrawAxis();

	// if want to reposition the single relevant stat box
/*	c1->Update();
	TPaveStats *s1 = (TPaveStats*)invMass[0]->FindObject("stats");
	s1->SetY1NDC(0.9); //new y start position
	s1->SetY2NDC(1.0); //new y end position
*/
	// throw away (far) the stat boxes of the 1st 2 histos
	c1->Update();
        TPaveStats *s1up = (TPaveStats*)invMass_envelope_up->FindObject("stats");
        s1up->SetY1NDC(10.); //new y start position
        s1up->SetY2NDC(10.); //new y end position
	c1->Update();
	TPaveStats *s1down = (TPaveStats*)invMass_envelope_down->FindObject("stats");
	s1down->SetY1NDC(10.); //new y start position
	s1down->SetY2NDC(10.); //new y end position

	cout << "invMass envelopes...ok" << endl;
	//----------------------------------------------------------------------------------------------------------



	//----------------------------------------------------------------------------------------------------------
	// do the same for the ***pT distribution***
	// allocate arrays for the envelopes
	Double_t *max_Rapidity = (Double_t *)malloc(rapnBins*sizeof(Double_t));
	Double_t *min_Rapidity = (Double_t *)malloc(rapnBins*sizeof(Double_t));

	// declare the up/down envelopes histos
	TH1 *Rapidity_envelope_up = new TH1D("Rapidity upEnvelope", "Rapidity upEnvelope", rapnBins, rapMin, rapMax);
	Rapidity_envelope_up->SetLineWidth(2);
	Rapidity_envelope_up->SetLineColor(kGreen);
	Rapidity_envelope_up->SetFillColor(3); // green
	Rapidity_envelope_up->SetXTitle("Rapidity(mu-,mu+)");
	Rapidity_envelope_up->SetYTitle("Events");
	Rapidity_envelope_up->SetTitle("Muon pairs rapidity distribution: PDFs Uncertainty");

	TH1 *Rapidity_envelope_down = new TH1D("Rapidity downEnvelope", "Rapidity downEnvelope", rapnBins, rapMin, rapMax);
	Rapidity_envelope_down->SetLineWidth(2);
	Rapidity_envelope_down->SetLineColor(kGreen);
	Rapidity_envelope_down->SetFillColor(10); // white

	// initialize arrays for the envelopes
	for (int j=0 ; j<(int)rapnBins ; j++) { max_Rapidity[j] = min_Rapidity[j] = 0.; }
//	for (int j=0 ; j<(int)rapnBins ; j++) { max_Rapidity[j] = 0.;    min_Rapidity[j] = 100000.; }	

	// loop over sets
	for (int f=0 ; f<PDFSETS ; f++) {
		n=0;
		while (n<=(int)rapnBins) {
			binContent = (Double_t)Rapidity[f]->GetBinContent(n);
			centralSetBinContent = (Double_t)Rapidity[0]->GetBinContent(n);
			diff = centralSetBinContent - binContent;
			if (diff<0) { max_Rapidity[n] += diff*diff; }
			else        { min_Rapidity[n] += diff*diff; }
//			if (binContent > max_Rapidity[n]) { max_Rapidity[n] = binContent; }
//			if (binContent < min_Rapidity[n]) { min_Rapidity[n] = binContent; }
			n++;
		}
	}

	// fill the envelopes histos
        for (int j=0 ; j<(int)rapnBins ; j++) {
		up = (Double_t)Rapidity[0]->GetBinContent(j) + (Double_t)sqrt(max_Rapidity[j]);
		down = (Double_t)Rapidity[0]->GetBinContent(j) - (Double_t)sqrt(min_Rapidity[j]);
		if (down<0.) { down = 0.; }
//		up = (Double_t)Rapidity[0]->GetBinContent(j) + (Double_t)max_Rapidity[j];
//		down = (Double_t)Rapidity[0]->GetBinContent(j) - (Double_t)min_Rapidity[j];
	
                Rapidity_envelope_up->SetBinContent((Int_t)j,up);
                Rapidity_envelope_down->SetBinContent((Int_t)j,down);
        }

	// legends
	TLegend *leg2 = new TLegend(0.65,0.68,0.98,0.88);
	leg2->AddEntry(Rapidity[0],"Central Prediction [cteq6m.LHpdf]", "L");
	leg2->AddEntry(Rapidity_envelope_up,"Uncertainty Band [ cteq6.LHpdf(1->40)]");
	leg2->SetFillColor(5);

	c1->cd(2);
	gStyle->SetOptStat(11);
	Rapidity_envelope_up->Draw();
	Rapidity_envelope_down->Draw("sames");
	Rapidity[0]->DrawCopy("e1sames");
        leg2->Draw("sames");
	//paveText->Draw("sames");
	gPad->RedrawAxis();

	// if want to reposition the single relevant stat box
/*	c1->Update();
	TPaveStats *s2 = (TPaveStats*)Rapidity[0]->FindObject("stats");
	s2->SetY1NDC(0.9); //new y start position
	s2->SetY2NDC(1.0); //new y end position
*/
	// throw away (far) the stat boxes of the 1st 2 histos
	c1->Update();
	TPaveStats *s2up = (TPaveStats*)Rapidity_envelope_up->FindObject("stats");
	s2up->SetY1NDC(10.); //new y start position
	s2up->SetY2NDC(10.); //new y end position
	c1->Update();
	TPaveStats *s2down = (TPaveStats*)Rapidity_envelope_down->FindObject("stats");
	s2down->SetY1NDC(10.); //new y start position
	s2down->SetY2NDC(10.); //new y end position

	cout << "Rapidity envelopes...ok" << endl;
	//----------------------------------------------------------------------------------------------------------

	
	// finalize the graphics
        c1->Update();
	c1->Print(rootFileName.c_str(),"root");
	c1->Print(epsFileName.c_str());
	histFile->Write();

	// Done.
	return 0;
}

